# Generated by Django 5.2.9 on 2025-12-09 08:52

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('wbhuman_resources', '0022_remove_review_editable_mode'),
    ]

    operations = [
        migrations.AlterField(
            model_name='employeehumanresource',
            name='weekly_off_periods',
            field=models.ManyToManyField(related_name='employees_off', through='wbhuman_resources.EmployeeWeeklyOffPeriods', through_fields=('employee', 'period'), to='wbhuman_resources.defaultdailyperiod', verbose_name='Weekly off periods'),
        ),
    ]
