# Generated by Django 5.0.3 on 2024-04-11 08:24

import colorfield.fields
from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("wbhuman_resources", "0020_alter_employeeyearbalance_year_alter_review_year"),
    ]

    operations = [
        migrations.AlterField(
            model_name="position",
            name="color",
            field=colorfield.fields.ColorField(default="#FF0000", image_field=None, max_length=25, samples=None),
        ),
    ]
